<?php
/**
 * Daisycon Conversion Pixel for Prestashop
 *
 * Copyright (c) 2021 Daisycon (http://www.daisycon.com)
 *
 */

class DaisyconConversionPixel extends Module
{

	public function __construct()
	{
		$this->name = 'daisyconconversionpixel';
		$this->tab = 'administration';
		$this->version = '2.6';

		parent::__construct();

		// Added for translations
		$this->page = basename(__FILE__, '.php');
		$this->displayName = 'Daisycon Conversion Pixel';
		$this->description = 'Adding the Daisycon conversion pixel to the order confirmation page.';
	}

	public function install()
	{
		if (parent::install() && $this->registerHook('displayOrderConfirmation'))
		{
			return true;
		}

		return false;
	}

	public function uninstall()
	{
		if (parent::uninstall())
		{
			return (Configuration::deleteByName('daisyconProgramId') && Configuration::deleteByName('daisyconMatchingDomain') && Configuration::deleteByName('daisyconTax'));
		}

		return false;
	}

	/**
	 * @return mixed
	 */
	public function getContent()
	{
		$html = '';
		$error = false;

		if (true === isset($_POST['submitModule']))
		{
			if (true === isset($_POST['daisyconProgramId']) && true === isset($_POST['daisyconProgramId']))
			{
				if (true === empty($_POST['daisyconProgramId']) || 1 !== preg_match('#^[1-9][0-9]*$#', $_POST['daisyconProgramId']))
				{
					$error = '
						<div class="failed" style="height:50px;">
							<div style="margin-top:15px;background: #ffd7a4;padding: 20px;">Program ID is empty or incorrect</div>
						</div>
					';
				}

				if (true === empty($_POST['daisyconMatchingDomain']))
				{
					$error = '
						<div class="failed" style="height:50px;">
							<div style="margin-top:15px;background: #ffd7a4;padding: 20px;">Matching Domain is empty</div>
						</div>
					';
				}

				if (false === $error)
				{
					Configuration::updateValue('daisyconProgramId', $_POST['daisyconProgramId']);
					Configuration::updateValue('daisyconMatchingDomain', $_POST['daisyconMatchingDomain']);
					Configuration::updateValue('daisyconTax', false === isset($_POST['daisyconTax']) ? 'yes' : $_POST['daisyconTax']);

					$html .= '
						<div class="confirm" style="height:50px;">
							<div style="margin-top:15px;background: #d3ffe7;padding: 20px;">Data has been saved! The Daisycon Conversion Pixel is now active. Please talk to your contact at Daisycon for further instructions.</div>
						</div>
					';
				}
				else
				{
					$html .= $error;
				}
			}
		}

		$html .= '
			<h2>' . $this->displayName . '</h2>
			<p>You have received an unique <a href="https://faq-advertiser.daisycon.com/hc/nl/articles/206902879-Wat-is-mijn-programma-ID-en-waar-vind-ik-die-">Daisycon campaign / program ID</a> and a matching domain. If you have not received this, please talk to your contact at Daisycon or click <a href="https://faq-advertiser.daisycon.com/hc/nl/articles/206902879-Wat-is-mijn-programma-ID-en-waar-vind-ik-die-">here</a> for more information.</p>

			<p>To add the Daisycon Conversion Pixel to your webshop, please fill out the form below.</p>
		';

		if (false !== Configuration::get('daisyconProgramId'))
		{
			$html .= '
				<h2>Current settings</h2>
				- Program ID: ' . Tools::safeOutput(Configuration::get('daisyconProgramId')) . ' <br>
				- Matching Domain: ' . Tools::safeOutput(Configuration::get('daisyconMatchingDomain')) . ' <br>
				- TAX included: ' . Tools::safeOutput(Configuration::get('daisyconTax')) . ' <br>

				<h2>Modify current settings</h2>
			';
		}

		$message = false !== $error ? ' style="border:1px solid red;"' : '';

		$html .= '
			<form action="' . Tools::htmlentitiesutf8($_SERVER['REQUEST_URI']) . '" method="post">
				<fieldset>
					<p>
						<label for="daisyconProgramId">Program ID:</label>
						<input type="text" id="daisyconProgramId" name="daisyconProgramId" value="' . Tools::safeOutput(Configuration::get('daisyconProgramId')) . '"'. $message . ' />
					</p>

					<p>
						<label for="daisyconMatchingDomain">Matching Domain:</label>
						<select id="daisyconMatchingDomain" name="daisyconMatchingDomain"'. $message . '>
							<option value="">Select Matching Domain</option>
		';

		$domains = [
			'at19.net',
			'ds1.nl',
			'dt51.net',
			'dt61.net',
			'fr135.net',
			'hs82.net',
			'jf79.net',
			'lt45.net',
			'mt74.net',
			'ndt5.net',
		];

		foreach ($domains as $domain)
		{
			$selected = '';
			if ($domain === Configuration::get('daisyconMatchingDomain'))
			{
				$selected = ' selected';
			}

			$html .= '<option value="' . $domain . '"' . $selected . '>' . $domain . '</option>';
		}

		$html .= '
						</select>
					</p>

					<p>
						<label for="daisyconTax">Include TAX:</label>
						<select id="daisyconTax" name="daisyconTax"' . $message . '>
							<option value="yes">Yes (default)</option>
							<option value="no">No</option>
						</select>
					</p>

					<div class="margin-form">
						<input type="submit" name="submitModule" value="Update settings" class="button" />
					</div>
				</fieldset>
			</form>
		';

		return $html;
	}

	/**
	 * @param array $params
	 * @return mixed
	 */
	public function hookOrderConfirmation(array $params)
	{
		$cartId = false;

		if (true === isset($params['order']->id_cart))
		{
			$cartId = $params['order']->id_cart;
		}
		else if (true === isset($_GET['id_cart']) && true === is_numeric($_GET['id_cart']))
		{
			$cartId = $_GET['id_cart'];
		}

		if (false !== $cartId)
		{
			// Get cart based on order information
			$cartOrder = new Cart($cartId);
			$orderProducts = $cartOrder->getProducts();

			// Get currency
			$currency = new Currency(intval($cartOrder->id_currency));

			// Variables to use
			$loop = 0;
			$productPart = '';
			$productVariablesList = [];
			foreach ($orderProducts as $products => $product)
			{
				$productWithTax = $product['price_with_reduction'] ?? $product['price_with_reduction_tax_incl'] ?? 0.0;
				$productWithoutTax = $product['price_with_reduction_without_tax']?? $product['price_with_reduction_tax_excl']?? 0.0;
				$price = 'no' !== Configuration::get('daisyconTax') ? $productWithTax : $productWithoutTax;

				$productVariablesList[] = [
					'pn'  => urlencode($product['name']),
					'a'   => number_format($price * number_format($product['quantity'], 0, '.', ''), 2, '.', ''),
					'iv'  => urlencode($product['name']),
					'r'   => number_format($price * number_format($product['quantity'], 0, '.', ''), 2, '.', ''),
					'qty' => number_format($product['quantity'], 0, '.', ''),
					'cc'  => urlencode($product['category']),
				];
				$loop++;
			}

			// Sort Product variables by price (a) and generate url string
			usort(
				$productVariablesList,
				function ($a, $b) {
					return strcmp($a['a'], $b['a']);
				}
			);
			$productPart = $this->getProductVariablesUrl($productVariablesList);

			if (true === isset($params['order']->id) && false === empty($params['order']->id))
			{
				$transactionId = $params['order']->id;
			}
			else
			{
				$transactionId = false === empty($_GET['id_order']) ? $_GET['id_order'] : date('d-m-Y-H:i:s');
			}

			if (true === isset($currency->iso_code) && false === empty($currency->iso_code))
			{
				$currencyCode = $currency->iso_code;
			}
			else
			{
				$currencyCode = '';
			}

			$cartRules = $cartOrder->getCartRules();

			$variables = [
				'ci'  => Configuration::get('daisyconProgramId'),
				'ti'  => (string)$transactionId,
				'src' => 'prestashop-' . $this->version,
				'cur' => $currencyCode,
				'np'  => $loop,
				'pr'  => false === empty($cartRules) ? $cartRules[0]['code'] : '',
			];

			if (false === Configuration::get('daisyconMatchingDomain'))
			{
				$domain = 'lt45.net';
			}
			else
			{
				$domain = Configuration::get('daisyconMatchingDomain');
			}

			$url = 'https://' . $domain . '/t/';
			$glue = '?';

			foreach ($variables as $key => $value)
			{
				$url .= $glue . $key . '=' . urlencode($value);
				$glue = '&';
			}

			$url = $url . $productPart;

			global $smarty;
			$smarty->assign([
				'url' => $url,
			]);

			return $this->display(__FILE__, 'daisyconconversionpixel.tpl');
		}
	}

	/**
	 * Generate product variables Url string
	 *
	 * @param $productVariablesList
	 * @return string
	 */
	private function getProductVariablesUrl($productVariablesList): string
	{
		if (true === empty($productVariablesList))
		{
			return '';
		}

		$productPart = '';

		foreach ($productVariablesList as $productVariables)
		{
			$productPart .= '&p[]=';

			foreach ($productVariables as $key => $value)
			{
				$productPart .= "{" . $key . ":" . urlencode($value) . "}";
			}
		}

		return $productPart;
	}

}
